package com.huimv.farm.damsubsidy.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.damsubsidy.common.token.TokenSign;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.entity.*;
import com.huimv.farm.damsubsidy.mapper.*;
import com.huimv.farm.damsubsidy.service.IBillLandingInspectionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 落地检 服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class BillLandingInspectionServiceImpl extends ServiceImpl<BillLandingInspectionMapper, BillLandingInspection> implements IBillLandingInspectionService {

    @Resource
    private BillLandingInspectionMapper billLandingInspectionMapper;
    @Resource
    private BillIsolatedMapper isolatedMapper;
    @Resource
    private BillBuyInfoMapper buyInfoMapper;
    @Resource
    private BillSubsidyMapper subsidyMapper;
    @Resource
    private SysUserMapper sysUserMapper;

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        Integer type = Integer.parseInt(paramsMap.get("type"));//信息类型
        String date = paramsMap.get("data");//状态
        String pageSize = paramsMap.get("pageSize");
        String pageNum = paramsMap.get("pageNum");
        if (pageSize == null || pageSize == "") {
            pageSize = "10";
        }
        if (pageNum == null || pageNum == "") {
            pageNum = "1";
        }

        System.out.println(type);
        if (type == 1) { //检疫申请
            QueryWrapper<BillLandingInspection> queryWrapper = new QueryWrapper<>();
            if (date == null || date == "") {
                queryWrapper.orderByDesc("create_time");
            } else {
                queryWrapper.orderByDesc("create_time").eq("inspection_sch", date);
            }
            Page<BillLandingInspection> billLandingInspectionPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, billLandingInspectionMapper.selectPage(billLandingInspectionPage, queryWrapper));
        }
        //交易信息
        if (type == 2) {
            QueryWrapper<BillBuyInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time");
            Page<BillBuyInfo> buyInfoPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, buyInfoMapper.selectPage(buyInfoPage, queryWrapper));
        }
        //补贴申请
        if (type == 3) {
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time");
            Page<BillSubsidy> subsidyPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, subsidyMapper.selectPage(subsidyPage, queryWrapper));
        }
        //验收信息
        if (type == 4) {
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time");
            Page<BillSubsidy> subsidyPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, subsidyMapper.selectPage(subsidyPage, queryWrapper));
        }
        //支付信息
        if (type == 5) {
            QueryWrapper<BillSubsidy> queryWrapper = new QueryWrapper<>();
            queryWrapper.orderByDesc("create_time").eq("subsidy_sch", 3);
            Page<BillSubsidy> subsidyPage = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
            return new Result(ResultCode.SUCCESS, subsidyMapper.selectPage(subsidyPage, queryWrapper));
        }
        return new Result(ResultCode.FAIL);
    }

    @Override
    public Result listById(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        Integer id = Integer.valueOf(paramsMap.get("id"));
        String num = paramsMap.get("num");//检疫证号
        Integer type = Integer.parseInt(paramsMap.get("type"));//消息类型
        JSONObject jsonObject = new JSONObject();
        if (type == 1) {
            //检疫
            jsonObject.put("data1", billLandingInspectionMapper.selectById(id));
            jsonObject.put("data2", isolatedMapper.selectOne(new QueryWrapper<BillIsolated>().eq("inspection_num", num)));
        }
        if (type == 2) {
            //交易
            jsonObject.put("data1", buyInfoMapper.selectById(id));
            jsonObject.put("data2", billLandingInspectionMapper.selectOne(new QueryWrapper<BillLandingInspection>().eq("inspection_num", num)));
        }
        if (type == 3) {
            //补贴
            jsonObject.put("data1", subsidyMapper.selectById(id));
            jsonObject.put("data2", "");
        }
        if (type == 4) {
            //验收
            jsonObject.put("data1", subsidyMapper.selectById(id));
            jsonObject.put("data2", "");
        }
        if (type == 5) {
            //支付
            jsonObject.put("data1", subsidyMapper.selectById(id));
            jsonObject.put("data2", "");
        }
        return new Result(ResultCode.SUCCESS, jsonObject);

    }

    @Override
    public Result addPay(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String url = paramsMap.get("url");
        BillSubsidy billSubsidy = subsidyMapper.selectById(id);
        billSubsidy.setPayImgUrl(url);
        billSubsidy.setIsPay(2);
        subsidyMapper.updateById(billSubsidy);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result addApplication(HttpServletRequest httpServletRequest, BillLandingInspection billLandingInspection) {
        QueryWrapper<BillLandingInspection> landingInspectionQueryWrapper = new QueryWrapper<>();
        landingInspectionQueryWrapper.eq("inspection_num", billLandingInspection.getInspectionNum());
        if (this.count(landingInspectionQueryWrapper) != 0) {
            return new Result(10001, "存在相同检疫证号！", false);
        }
        billLandingInspection.setInspectionName(TokenSign.getUserName(httpServletRequest) + "的落地检疫申请");
        billLandingInspection.setCreateTime(DateTime.now());
        billLandingInspection.setCreateUser(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setInspectionType(0);
        if (this.save(billLandingInspection)) {
            return Result.SUCCESS();
        } else {
            return Result.FAIL();
        }
    }

    @Override
    public Result cancelApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
//        BillLandingInspection landingInspection = this.getById(billLandingInspection);
//        landingInspection.setInspectionSch(5);
//        if (this.updateById(landingInspection)) {
//            return Result.SUCCESS();
//        }else {
//            return Result.FAIL();
//        }
        String id = paramsMap.get("id");
        if (this.removeById(id)) {
            return Result.SUCCESS();
        }else {
            return Result.ERROR();
        }
    }

    @Override
    public Result listApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String pageSize = paramsMap.get("pageSize");
        String pageNum = paramsMap.get("pageNum");
        String startTime = paramsMap.get("startTime");
        String endTime = paramsMap.get("endTime");
        String inspectionSch = paramsMap.get("inspectionSch");
        String userName = TokenSign.getUserName(httpServletRequest);
        Integer userType = sysUserMapper.selectOne(new QueryWrapper<SysUser>().eq("user_name", userName)).getUserType();


        QueryWrapper<BillLandingInspection> landingInspectionQueryWrapper = new QueryWrapper<>();
        if (userType == 2 || userType == 3){
            landingInspectionQueryWrapper.eq("create_user",userName);
        }

        if (startTime != null) {
            startTime = startTime + " 00:00:00";
            landingInspectionQueryWrapper.ge("create_time",startTime);
        }
        if (endTime != null) {
            endTime = endTime + " 23:59:59";
            landingInspectionQueryWrapper.le("create_time",endTime);
        }
        if (ObjectUtil.isNotEmpty(inspectionSch)){
            landingInspectionQueryWrapper.eq("inspection_sch",inspectionSch);
        }
        landingInspectionQueryWrapper.orderByDesc("create_time");
        Page<BillLandingInspection> page = new Page(Integer.parseInt(pageNum),Integer.parseInt(pageSize));
        return new Result(ResultCode.SUCCESS,billLandingInspectionMapper.selectPage(page,landingInspectionQueryWrapper));
    }

    @Override
    public Result rejectApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        String rejectReason = paramsMap.get("rejectReason");
        BillLandingInspection billLandingInspection = this.getById(id);
        billLandingInspection.setInspectionSch(5);
        billLandingInspection.setRejectReason(rejectReason);
        billLandingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        billLandingInspection.setReviewedTime(DateTime.now());
        this.updateById(billLandingInspection);
        return Result.SUCCESS();
    }

    @Override
    public Result agreeApplication(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String id = paramsMap.get("id");
        BillLandingInspection landingInspection = this.getById(id);
        landingInspection.setInspectionSch(2);
        landingInspection.setReviewed(TokenSign.getUserName(httpServletRequest));
        landingInspection.setReviewedTime(DateTime.now());
        if (this.updateById(landingInspection)){
            return Result.SUCCESS();
        }else {
            return Result.ERROR();
        }
    }
}
