package com.huimv.farm.damsubsidy.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.huimv.farm.damsubsidy.common.utils.Result;
import com.huimv.farm.damsubsidy.common.utils.ResultCode;
import com.huimv.farm.damsubsidy.entity.PovertyUserRegist;
import com.huimv.farm.damsubsidy.entity.SysUser;
import com.huimv.farm.damsubsidy.mapper.PovertyUserRegistMapper;
import com.huimv.farm.damsubsidy.mapper.SysUserMapper;
import com.huimv.farm.damsubsidy.service.ISysUserService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author author
 * @since 2023-04-26
 */
@Service
public class SysUserServiceImpl extends ServiceImpl<SysUserMapper, SysUser> implements ISysUserService {

    @Autowired
    private SysUserMapper userMapper;
    @Autowired
    private PovertyUserRegistMapper povertyUserRegistMapper;
    @Override
    public Result loginMultilevel(HttpServletRequest httpServletRequest, Map<String,String> paramsMap) {
        //TODO 验证码接口暂时不写，验证码验证接口写在此接口之前，这里默认验证码已经正确！
        String phone = paramsMap.get("phone");
        if (!phone.matches("1[3-9]\\d{9}")) {
            return new Result(ResultCode.FAIL, "手机号格式不正确");
        }
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", phone);
        SysUser sysUser = userMapper.selectOne(queryWrapper);
        if (ObjectUtil.isEmpty(sysUser)) {
            return new Result(ResultCode.DATA_NULL, "用户暂未注册，请注册！");
        } else {
            return new Result(ResultCode.SUCCESS, sysUser);
        }
    }
    @Override
    public Result loginMultilevelClient(String accountName, String password) {

        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        wrapper.eq("account_num",accountName).eq("password",password);
        SysUser accountMultilevel = userMapper.selectOne(wrapper);

        if (ObjectUtil.isEmpty(accountMultilevel)){
            return new Result(10001, "账号不存在或密码错误.", false);
        }

        Integer accoutType = accountMultilevel.getAccoutType();

        if ( accoutType!= 1  ){
            return new Result(10002, "账号未启用.", false);
        }
        return new Result(ResultCode.SUCCESS,accountMultilevel);
    }


    @Override
    public Result add(HttpServletRequest httpServletRequest,Map<String,String> paramsMap) {

        SysUser user = new SysUser();
        user.setUserType(Integer.parseInt(paramsMap.get("type")));
        user.setUserName(paramsMap.get("userName"));
        user.setIdCard(paramsMap.get("idCard"));
        user.setPhone(paramsMap.get("phone"));
        user.setIdCardFrontUrl(paramsMap.get("idCordFrontUrl"));
        user.setIdCardBackUrl(paramsMap.get("idCordBackUrl"));
        user.setCreateTime(new Date());
        user.setCreateUser(paramsMap.get("userName"));
        if (ObjectUtil.isNotEmpty(paramsMap.get("address"))) {
            user.setAddress(paramsMap.get("address"));
        }
        if (Integer.parseInt(paramsMap.get("type")) == 3) {
            QueryWrapper<PovertyUserRegist> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("XM", paramsMap.get("userName")).eq("ZJHM", paramsMap.get("idCard"));
            PovertyUserRegist povertyUserRegist = povertyUserRegistMapper.selectOne(queryWrapper);
            if (ObjectUtil.isEmpty(povertyUserRegist)) {
                user.setFarmType(2);
            } else if (povertyUserRegist.getIsDelete() == 1) {
                user.setFarmType(1);
            } else {
                user.setFarmType(0);
            }
        }
        user.setAccoutType(0);
        user.setIsRealName(1);

        QueryWrapper<SysUser> queryWrapper1 = new QueryWrapper<>();
        queryWrapper1.eq("user_name", paramsMap.get("userName")).eq("id_card", paramsMap.get("idCord"));
        SysUser user2 = userMapper.selectOne(queryWrapper1);
        if (ObjectUtil.isNotEmpty(user2)) {
            return new Result(ResultCode.FAIL, "");
        }
        userMapper.insert(user);
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", paramsMap.get("phone"));
        SysUser user1 = userMapper.selectOne(queryWrapper);
        return new Result(ResultCode.SUCCESS,user1);
    }

    @Override
    public Result edit(HttpServletRequest httpServletRequest,SysUser user) {
        userMapper.updateById(user);
        return new Result(ResultCode.SUCCESS,"修改成功！");
    }

    @Override
    public Result list(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("phone", paramsMap.get("phone"));
        return new Result(ResultCode.SUCCESS,userMapper.selectOne(queryWrapper));
    }

    @Override
    public Result findPhone(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String userName = paramsMap.get("userName");
        String idCord = paramsMap.get("idCord");
        String phone = paramsMap.get("phone");
        if (!phone.matches("1[3-9]\\d{9}")) {
            return new Result(ResultCode.FAIL, "手机号格式不正确");
        }
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_name", userName).eq("id_card", idCord);
        SysUser user = userMapper.selectOne(queryWrapper);
        return new Result(ResultCode.SUCCESS, user.getPhone());
    }

    @Override
    public Result listPc(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String date = paramsMap.get("date");
        String word = paramsMap.get("word");
        String role = paramsMap.get("role");
        String pageSize = paramsMap.get("pageSize");
        String pageNum = paramsMap.get("pageSize");

        if (pageSize == null || pageSize == "") {
            pageSize = "10";
        }
        if (pageNum == null || pageNum == "") {
            pageNum = "1";
        }
        if (date != "") {
            date = date + " 00:00:00";
        }

        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("accout_type", 0).ge("create_time", date)
                .eq("user_type", role).like("user_name", word).or().like("phone", word)
                .or().like("id_card", word);

        Page<SysUser> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        Page<SysUser> sysUserPage = userMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,sysUserPage);
    }

    @Override
    public Result editState(HttpServletRequest httpServletRequest, SysUser user) {
        userMapper.updateById(user);
        return new Result(ResultCode.SUCCESS);
    }

    @Override
    public Result listPc2(HttpServletRequest httpServletRequest, Map<String, String> paramsMap) {
        String date = paramsMap.get("startTime");
        String word = paramsMap.get("word");
        String role = paramsMap.get("role");
        String pageSize = paramsMap.get("pageSize");
        String pageNum = paramsMap.get("pageSize");

        if (pageSize == null || pageSize == "") {
            pageSize = "10";
        }
        if (pageNum == null || pageNum == "") {
            pageNum = "1";
        }
        if (date != "") {
            date = date + " 00:00:00";
        }
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("accout_type", 1).or().eq("accout_typr",4).ge("create_time", date)
                .eq("user_type", role).like("user_name", word).or().like("phone", word)
                .or().like("id_card", word);

        Page<SysUser> page = new Page<>(Integer.parseInt(pageNum), Integer.parseInt(pageSize));
        Page<SysUser> sysUserPage = userMapper.selectPage(page, queryWrapper);
        return new Result(ResultCode.SUCCESS,sysUserPage);
    }

  
    
    //
    @Override
    public Result findUserAccount(String searchStr, int pageNum, int pageSize, Long userId) {

        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        wrapper.like("pids",userId).or().eq("id",userId);
        IPage<SysUser> page = this.page(new Page<>(pageNum, pageSize), wrapper);
        return new Result(ResultCode.SUCCESS,page);
    }

    @Override
    public Result add(SysUser sysUser, Integer userId) {

        int account = this.count(new QueryWrapper<SysUser>().eq("account_num", sysUser.getAccountNum()));
        if (account > 0 ){
            return new Result(10001,"该账号已经存在，请重新创建",false);

        }

        SysUser byId = this.getById(userId);


        sysUser.setPid(userId);
        sysUser.setPids(byId.getPids()+","+userId);
        userMapper.insert(sysUser);
        return  new Result( 10000,"添加成功",true);
    }

    @Override
    public Result setAccountStatus(List<Integer> ids, Integer status) {
        try {
            for (Integer id : ids) {
                SysUser account= new SysUser();
                account.setId(Long.valueOf(id));
                account.setAccoutType(status);
                this.updateById(account);
            }
        }catch (Exception e){
            return new Result(10001,"插入失败",false);
        }
        String message = "";
        if(status == 1){
            message = "你选择的账号已经启用。";
        }else{
            message = "你选择的账号已经暂停启用。";
        }
        return new Result(10000,message,true);
    }

    @Override
    public List<Long> getAllChildId(Long userId) {
        List<SysUser> pids = this.list(new QueryWrapper<SysUser>().like("pids", userId + ""));
        List<Long> list = new ArrayList<>();
        for (SysUser pid : pids) {
            list.add(pid.getId());
        }
        list.add(userId);
        return list;
    }

    @Override
    public Result listSubordinate(String searchStr, int pageNum, int pageSize, Long userId) {

        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        wrapper.like("pids",userId);
        if (StringUtils.isNotBlank(searchStr)){
            wrapper.like("user_name",searchStr);
        }

        IPage<SysUser> page = this.page(new Page<>(pageNum, pageSize), wrapper);
        return new Result(ResultCode.SUCCESS,page);
    }

    @Override
    public Result getLastFarmId(Integer userId) {
        SysUser sysUser = userMapper.selectById(userId);
        Integer lastFarmId = sysUser.getLastVeter();
        String farmIds = sysUser.getVeterIds();
        if (farmIds == null){
            return new Result(10001,"该账号没有绑定兽医站",true);
        }
        if (lastFarmId == null){
            String[] split = farmIds.split(",");
            String s = split[0];
            sysUser.setLastVeter(Integer.valueOf(s));
            userMapper.updateById(sysUser);
            return new Result(ResultCode.SUCCESS,s);
        }
        return new Result(ResultCode.SUCCESS,lastFarmId);
    }


}
